import contextlib
import json
import os
from copy import deepcopy
from typing import List, Optional, Union

import requests
from loguru import logger

from ssrspeed.config import ssrconfig
from ssrspeed.parser.clash import ClashParser
from ssrspeed.parser.conf import V2RayBaseConfigs, shadowsocks_get_config
from ssrspeed.parser.filter import NodeFilter
from ssrspeed.parser.ss import (
    ParserShadowsocksBasic,
    ParserShadowsocksD,
    ParserShadowsocksSIP002,
)
from ssrspeed.parser.ssr import ParserShadowsocksR
from ssrspeed.parser.trojan import TrojanParser
from ssrspeed.parser.v2ray import ParserV2RayN, ParserV2RayQuantumult
from ssrspeed.type.node import NodeShadowsocks, NodeShadowsocksR, NodeTrojan, NodeV2Ray
from ssrspeed.util import b64plus

PROXY_SETTINGS = ssrconfig["proxy"]
LOCAL_ADDRESS = ssrconfig["localAddress"]
LOCAL_PORT = ssrconfig["localPort"]
TIMEOUT = 10

TMP_DIR = ssrconfig["path"]["tmp"]
if not os.path.exists(TMP_DIR):
    os.makedirs(TMP_DIR)
TEST_TXT = f"{TMP_DIR}test.txt"


class UniversalParser:
    def __init__(self):
        self.__nodes: list = []
        self.__ss_base_cfg: dict = shadowsocks_get_config(
            LOCAL_ADDRESS, LOCAL_PORT, TIMEOUT
        )

    @staticmethod
    def web_config_to_node(
        configs: List[dict],
    ) -> List[
        Union[
            Optional[NodeShadowsocks], Optional[NodeShadowsocksR], Optional[NodeV2Ray],
        ]
    ]:
        result: list = []
        for _config in configs:
            _type = _config.get("type", "N/A")
            if _type == "Shadowsocks":
                result.append(NodeShadowsocks(_config["config"]))
            elif _type == "ShadowsocksR":
                result.append(NodeShadowsocksR(_config["config"]))
            elif _type == "V2Ray":
                result.append(NodeV2Ray(_config["config"]))
            elif _type == "Trojan":
                result.append(NodeTrojan(_config["config"]))
            else:
                logger.warning(f"Unknown node type: {_type}")
        return result

    @property
    def nodes(self) -> list:
        return deepcopy(self.__nodes)

    def __get_ss_base_config(self) -> dict:
        return deepcopy(self.__ss_base_cfg)

    def __clean_nodes(self):
        self.__nodes.clear()

    def set_nodes(self, nodes: list):
        self.__clean_nodes()
        self.__nodes = nodes

    def set_group(self, group: str):
        tmp_nodes = deepcopy(self.__nodes)
        self.__clean_nodes()
        for node in tmp_nodes:
            if group:
                node.update_config({"group": group})
            self.__nodes.append(node)

    def parse_links(
        self, links: list
    ) -> List[
        Union[
            Optional[NodeShadowsocks],
            Optional[NodeShadowsocksR],
            Optional[NodeV2Ray],
            Optional[NodeTrojan],
        ]
    ]:
        # Single link parse
        result: list = []
        for link in links:
            link = link.replace("\r", "")
            node: Union[
                Optional[NodeShadowsocks],
                Optional[NodeShadowsocksR],
                Optional[NodeV2Ray],
                Optional[NodeTrojan],
            ] = None

            if link[:5] == "ss://":
                # Shadowsocks
                cfg = None
                try:
                    pssb = ParserShadowsocksBasic(self.__get_ss_base_config())
                    cfg = pssb.parse_single_link(link)
                except ValueError:
                    pssip002 = ParserShadowsocksSIP002(self.__get_ss_base_config())
                    cfg = pssip002.parse_single_link(link)
                if cfg:
                    node = NodeShadowsocks(cfg)
                else:
                    logger.warning(f"Invalid shadowsocks link {link}")

            elif link[:6] == "ssr://":
                # ShadowsocksR
                pssr = ParserShadowsocksR(self.__get_ss_base_config())
                if cfg := pssr.parse_single_link(link):
                    node = NodeShadowsocksR(cfg)
                else:
                    logger.warning(f"Invalid shadowsocksR link {link}")

            elif link[:8] == "vmess://":
                # Vmess link (V2RayN and Quan)
                # V2RayN Parser
                cfg = None
                logger.info("Try V2RayN Parser.")
                pv2rn = ParserV2RayN()
                with contextlib.suppress(ValueError):
                    cfg = pv2rn.parse_subs_config(link)
                if not cfg:
                    # Quantumult Parser
                    logger.info("Try Quantumult Parser.")
                    pq = ParserV2RayQuantumult()
                    with contextlib.suppress(ValueError):
                        cfg = pq.parse_subs_config(link)
                if not cfg:
                    logger.error(f"Invalid vmess link: {link}")
                else:
                    gen_cfg = V2RayBaseConfigs.generate_config(
                        cfg, LOCAL_ADDRESS, LOCAL_PORT
                    )
                    node = NodeV2Ray(gen_cfg)

            elif link[:9] == "trojan://":
                cfg = None
                logger.info("Try Trojan Parser.")
                ptrojan = TrojanParser()
                with contextlib.suppress(ValueError):
                    cfg = ptrojan.parse_single_link(link)
                if cfg:
                    node = NodeTrojan(cfg)

            else:
                logger.warning(f"Unsupported link: {link}")

            if node:
                result.append(node)

        return result

    @staticmethod
    def __parse_clash(clash_cfg: str) -> list:
        result: list = []
        pc = ClashParser(shadowsocks_get_config(LOCAL_ADDRESS, LOCAL_PORT, TIMEOUT))
        pc.parse_config(clash_cfg)
        cfgs = pc.config_list
        for cfg in cfgs:
            if cfg["type"] == "ss":
                result.append(NodeShadowsocks(cfg["config"]))
            elif cfg["type"] == "vmess":
                result.append(
                    NodeV2Ray(
                        V2RayBaseConfigs.generate_config(
                            cfg["config"], LOCAL_ADDRESS, LOCAL_PORT
                        )
                    )
                )
            elif cfg["type"] == "trojan":
                result.append(NodeTrojan(cfg["config"]))

        return result

    def filter_nodes(self, **kwargs: list):
        fk = kwargs.get("fk", [])
        fgk = kwargs.get("fgk", [])
        frk = kwargs.get("frk", [])
        ek = kwargs.get("ek", [])
        egk = kwargs.get("egk", [])
        erk = kwargs.get("erk", [])
        nf = NodeFilter()
        self.__nodes = nf.filter_node(
            self.__nodes, fk=fk, fgk=fgk, frk=frk, ek=ek, egk=egk, erk=erk
        )

    def print_nodes(self):
        for item in self.nodes:
            logger.info(f'{item.config["group"]} - {item.config["remarks"]}')

    def read_subscription(self, urls: list):
        for url in urls:
            if not url:
                continue

            if (
                url.startswith("ss://")
                or url.startswith("ssr://")
                or url.startswith("vmess://")
                or url.startswith("trojan://")
            ):
                self.__nodes.extend(self.parse_links([url]))
                continue

            logger.info(f"Reading {url}")
            header = {
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
                "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36"
            }
            clash_ua = {"User-Agent": "Clash"}

            try:
                r = requests.get(url, headers=clash_ua, timeout=15)
                t = r.headers["subscription-userinfo"]
                dl = int(t[t.find("download") + 9 : t.find("total") - 2])
                _sum = dl
            except Exception:
                _sum = 0

            with open(TEST_TXT, "a+", encoding="utf-8") as test:
                test.write(f"{url}\n")
                test.write(f"{_sum}\n")

            if PROXY_SETTINGS["enabled"]:
                auth = ""
                if PROXY_SETTINGS["username"]:
                    auth = f'{PROXY_SETTINGS["username"]}:{PROXY_SETTINGS["password"]}@'
                proxy = f'socks5://{auth}{PROXY_SETTINGS["address"]}:{PROXY_SETTINGS["port"]}'
                proxies = {"http": proxy, "https": proxy}
                logger.info(f"Reading subscription via {proxy}")
                rep = requests.get(url, headers=header, timeout=15, proxies=proxies)
            else:
                rep = requests.get(url, headers=header, timeout=15)
            rep.encoding = "utf-8"
            res = rep.text

            parsed = False
            # Try ShadowsocksD Parser
            if res[:6] == "ssd://":
                parsed = True
                logger.info("Try ShadowsocksD Parser.")
                pssd = ParserShadowsocksD(
                    shadowsocks_get_config(LOCAL_ADDRESS, LOCAL_PORT, TIMEOUT)
                )
                cfgs = pssd.parse_subs_config(b64plus.decode(res[6:]).decode("utf-8"))
                for cfg in cfgs:
                    self.__nodes.append(NodeShadowsocks(cfg))
            if parsed:
                continue

            # Try base64 decode
            try:
                res = res.strip()
                links = (b64plus.decode(res).decode("utf-8")).split("\n")
                logger.debug("Base64 decode success.")
                self.__nodes.extend(self.parse_links(links))
                parsed = True
            except ValueError:
                logger.info("Base64 decode failed.")
            if parsed:
                continue

            # Try Clash Parser
            self.__nodes.extend(self.__parse_clash(res))

    def read_gui_config(self, filename: str):
        with open(filename, "r", encoding="utf-8") as f:
            raw_data = f.read()
        try:
            # Try Load as Json
            data = json.loads(raw_data)
            # Identification of proxy type
            # Shadowsocks(D)
            if (
                "subscriptions" in data
                or "serverSubscribes" not in data
                and "vmess" not in data
            ):
                pssb = ParserShadowsocksBasic(self.__get_ss_base_config())
                for cfg in pssb.parse_gui_data(data):
                    self.__nodes.append(NodeShadowsocks(cfg))
            # ShadowsocksR
            elif "serverSubscribes" in data:
                pssr = ParserShadowsocksR(self.__get_ss_base_config())
                for cfg in pssr.parse_gui_data(data):
                    self.__nodes.append(NodeShadowsocksR(cfg))
            # V2RayN
            else:
                pv2n = ParserV2RayN()
                cfgs = pv2n.parse_gui_data(data)
                for cfg in cfgs:
                    self.__nodes.append(
                        NodeV2Ray(
                            V2RayBaseConfigs.generate_config(
                                cfg, LOCAL_ADDRESS, LOCAL_PORT
                            )
                        )
                    )
        except json.JSONDecodeError:
            # Try Load as Yaml
            self.__nodes = self.__parse_clash(raw_data)



from .shadowsocks import Shadowsocks as ShadowsocksClient
from .shadowsocksr import ShadowsocksR as ShadowsocksRClient
from .trojan import Trojan as TrojanClient
from .v2ray import V2Ray as V2RayClient

__all__ = ["ShadowsocksClient", "ShadowsocksRClient", "TrojanClient", "V2RayClient"]



import json
import os

from flask import Flask, redirect, request  # ,render_template
from flask_cors import CORS
from loguru import logger
from werkzeug.utils import secure_filename

from ssrspeed.config import ssrconfig
from ssrspeed.core import SSRSpeedCore
from ssrspeed.type.error.webapi import FileNotAllowed, WebFileCommonError
from ssrspeed.util.web import get_post_data

ALLOWED_EXTENSIONS = {"json", "yml"}


def check_file_allowed(filename):
    return "." in filename and filename.rsplit(".", 1)[1] in ALLOWED_EXTENSIONS


def server_method(app, sc):
    @app.route("/", methods=["GET"])
    def index():
        return redirect("https://web1.ospf.in/", 301)

    """
    {
        "proxyType": "SSR",      // [SSR, SSR-C#, SS, V2RAY]
        "testMethod": "SOCKET",  // [SOCKET, SPEED_TEST_NET, FAST]
        "testMode": "",          // [ALL, TCP_PING, WEB_PAGE_SIMULATION]
        "subscriptionUrl": "",
        "colors": "origin",
        "sortMethod": "",        // [SPEED, REVERSE_SPEED, PING, REVERSE_PING]
        "include": [],
        "includeGroup": [],
        "includeRemark": [],
        "exclude": [],
        "excludeGroup": [],
        "excludeRemark": []
    }
    """

    @app.route("/getversion", methods=["GET"])
    def get_version():
        return json.dumps({"main": ssrconfig["VERSION"]})

    @app.route("/status", methods=["GET"])
    def status():
        return sc.web_get_status()

    @app.route("/readsubscriptions", methods=["POST"])
    def read_subscriptions():
        if request.method == "POST":
            data = get_post_data()
            if sc.web_get_status() == "running":
                return "running"
            subscription_url = data.get("url", "")
            return (
                json.dumps(sc.web_read_subscription(subscription_url))
                if subscription_url
                else "invalid url."
            )

        return "invalid method"

    @app.route("/readfileconfig", methods=["POST"])
    def read_file_config():
        if request.method != "POST":
            return "invalid method"

        if sc.web_get_status() == "running":
            return "running"
        ufile = request.files["file"]
        if ufile:
            if check_file_allowed(ufile.filename):
                filename = secure_filename(ufile.filename)
                tmp_filename = os.path.join(app.config["UPLOAD_FOLDER"], filename)
                ufile.save(tmp_filename)
                logger.info(f"Tmp config file saved as {tmp_filename}.")
                return json.dumps(sc.web_read_config_file(tmp_filename))
            logger.error(f"Disallowed file {ufile.filename}.")
            return FileNotAllowed.err_msg
        logger.error("File upload failed or unknown error.")
        return WebFileCommonError.err_msg

    @app.route("/getcolors", methods=["GET"])
    def get_colors():
        return json.dumps(sc.web_get_colors())

    @app.route("/start", methods=["POST"])
    def start_test():
        if request.method != "POST":
            return "invalid method"

        data = get_post_data()
        if sc.web_get_status() == "running":
            return "running"
        configs = data.get("configs", [])
        if not configs:
            return "No configs"
        test_method = data.get("testMethod", "ST_ASYNC")
        colors = data.get("colors", "origin")
        sort_method = data.get("sortMethod", "")
        test_mode = data.get("testMode", "")
        use_ssr_cs = data.get("useSsrCSharp", False)
        group = data.get("group", "")
        sc.web_setup(
            testMode=test_mode,
            testMethod=test_method,
            colors=colors,
            sortMethod=sort_method,
        )
        sc.clean_result()
        sc.web_set_configs(configs)
        if group:
            sc.set_group(group)
        sc.start_test(use_ssr_cs)
        return "done"

    @app.route("/getresults")
    def get_results():
        return json.dumps(sc.web_get_results())


def start_server(args, key_path):
    template_dir = key_path["templates"]
    static_dir = key_path["static"]
    upload_dir = key_path["uploads"]

    app = Flask(
        __name__,
        template_folder=template_dir,
        static_folder=static_dir,
        static_url_path="",
    )
    app.config["UPLOAD_FOLDER"] = upload_dir
    CORS(app)
    sc = SSRSpeedCore()
    server_method(app, sc)
    sc.web_mode = True
    if not os.path.exists(upload_dir):
        logger.warning(f" Upload folder {upload_dir} not found, creating.")
        os.makedirs(upload_dir)
    port = args.port or ssrconfig["web"]["port"]
    host = args.listen or ssrconfig["web"]["listen"]
    app.run(host=host, port=port, debug=args.debug, threaded=True)



from .nss import NodeShadowsocks
from .nssr import NodeShadowsocksR
from .ntrojan import NodeTrojan
from .nv2ray import NodeV2Ray

__all__ = ["NodeShadowsocks", "NodeShadowsocksR", "NodeTrojan", "NodeV2Ray"]



import os
import subprocess
import sys
from typing import Dict, Optional

from loguru import logger


class RequirementsCheck:
    def __init__(self, client_dir: str, databases_dir: str):
        self.__win_require: dict = {
            "Shadowsocks-libev": [
                f"{client_dir}shadowsocks-libev/obfs-local.exe",
                f"{client_dir}shadowsocks-libev/simple-obfs.exe",
                f"{client_dir}shadowsocks-libev/ss-local.exe",
                f"{client_dir}shadowsocks-libev/ss-tunnel.exe",
            ],
            "ShadowsocksR-libev": [
                f"{client_dir}shadowsocksr-libev/libssp-0.dll",
                f"{client_dir}shadowsocksr-libev/libwinpthread-1.dll",
                f"{client_dir}shadowsocksr-libev/libpcre-1.dll",
                f"{client_dir}shadowsocksr-libev/libcrypto-1_1.dll",
                f"{client_dir}shadowsocksr-libev/ssr-local.exe",
            ],
            "ShadowsocksR-C#": [f"{client_dir}shadowsocksr-win/shadowsocksr.exe"],
            "Trojan": [f"{client_dir}trojan/trojan.exe"],
            "V2Ray-Core": [
                f"{client_dir}v2ray-core/v2ctl.exe",
                f"{client_dir}v2ray-core/v2ray.exe",
            ],
            "GeoIP2 Databases": [
                f"{databases_dir}GeoLite2-City.mmdb",
                f"{databases_dir}GeoLite2-ASN.mmdb",
            ],
        }

        self.__linux_require: dict = {
            "Shadowsocks-libev": [
                f"{client_dir}shadowsocks-libev/simple-obfs",
                f"{client_dir}shadowsocks-libev/ss-local",
            ],
            "ShadowsocksR-Python": [f"{client_dir}shadowsocksr/shadowsocks/local.py"],
            "Trojan": [f"{client_dir}trojan/trojan"],
            "V2Ray-Core": [
                f"{client_dir}v2ray-core/v2ctl",
                f"{client_dir}v2ray-core/v2ray",
            ],
            "GeoIP2 Databases": [
                f"{databases_dir}GeoLite2-City.mmdb",
                f"{databases_dir}GeoLite2-ASN.mmdb",
            ],
        }

    def check(self, platform: str):
        if platform == "Windows":
            self.__checks(self.__win_require)
        elif platform in {"Linux", "MacOS"}:
            self.__linux_check(platform)
            self.__checks(self.__linux_require)
        else:
            logger.critical("Unsupported platform !")
            sys.exit(1)

    @staticmethod
    def __checks(requires: Optional[Dict[str, str]]):
        if requires is None:
            requires = {}
        for key in requires.keys():
            for require in requires[key]:
                logger.info(f"Checking {require}")
                if os.path.exists(require):
                    if os.path.isdir(require):
                        logger.warning(f"Requirement {require} not found !!!")
                else:
                    logger.warning(f"Requirement {require} not found !!!")

    def __linux_check(self, platform: str):
        if not self.__linux_check_libsodium(platform):
            logger.critical("Requirement libsodium not found !!!")
            sys.exit(1)
        self.__checks(self.__linux_require)

    #   self.__linux_check_shadowsocks()

    @staticmethod
    def __linux_check_libsodium(platform: str) -> bool:
        logger.info("Checking libsodium.")
        if platform == "MacOS":
            logger.warning(
                "MacOS does not support detection of libsodium, please ensure that libsodium is installed."
            )
            try:
                process = subprocess.Popen(
                    "brew info libsodium", shell=True, stdout=subprocess.PIPE
                )
                try:
                    out = process.communicate(timeout=15)[0]
                except subprocess.TimeoutExpired:
                    process.terminate()
                    out, errs = process.communicate()
                    logger.error(
                        out.decode("utf-8") + errs.decode("utf-8"), exc_info=True
                    )
                    return False
                logger.debug(f"brew info libsodium : {out!r}")
                if "Not installed\n" in out.decode("utf-8"):
                    logger.error("Libsodium not found.")
                    return False
                return True
            except Exception:
                logger.exception("")
                return False
        else:
            try:
                process = subprocess.Popen(
                    "ldconfig -p | grep libsodium", shell=True, stdout=subprocess.PIPE
                )
                try:
                    out = process.communicate(timeout=15)[0]
                except subprocess.TimeoutExpired:
                    process.terminate()
                    out, errs = process.communicate()
                    logger.error(
                        out.decode("utf-8") + errs.decode("utf-8"), exc_info=True
                    )
                    return False
                logger.debug(f"ldconfig : {out!r}")
                return "libsodium" in out.decode("utf-8")
            except Exception:
                logger.exception("")
                return False

    # @staticmethod
    # def __linux_check_shadowsocks() -> bool:
    #     sslibev = False
    #     simpleobfs = False
    #     for cmdpath in os.environ["PATH"].split(":"):
    #         if not os.path.isdir(cmdpath):
    #             continue
    #         for filename in os.listdir(cmdpath):
    #             if filename == "obfs-local":
    #                 logger.info(
    #                     f'Obfs-Local found {os.path.join(cmdpath, "obfs-local")}'
    #                 )
    #                 simpleobfs = True
    #             elif filename == "ss-local":
    #                 logger.info(
    #                     f'Shadowsocks-libev found {os.path.join(cmdpath, "ss-local")}'
    #                 )
    #                 sslibev = True
    #             if simpleobfs and sslibev:
    #                 break
    #         if simpleobfs and sslibev:
    #             break
    #     if not simpleobfs:
    #         logger.warning("Simple Obfs not found !!!")
    #     if not sslibev:
    #         logger.warning("Shadowsocks-libev not found !!!")
    #     return True if (simpleobfs and sslibev) else False



import contextlib
import json
from typing import Optional, Union

from loguru import logger

from ssrspeed.util import b64plus


class ParserV2RayN:
    def __init__(self):
        self.__decoded_configs = []

    @staticmethod
    def parse_subs_config(raw_link: str) -> Optional[dict]:
        link = raw_link[8:]
        link_decoded = b64plus.decode(link).decode("utf-8")
        try:
            _conf = json.loads(link_decoded)
        except json.JSONDecodeError:
            return None

        try:
            cfg_version = str(_conf.get("v", "1"))
            server = _conf["add"]
            port = int(_conf["port"])
            _type = _conf.get("type", "none")  # Obfs type
            uuid = _conf["id"]
            aid = int(_conf["aid"])
            net = _conf["net"]
            host = ""
            path = ""
            if cfg_version == "2":
                host = _conf.get(
                    "host", ""
                )  # http host, web socket host, h2 host, quic encrypt method
                path = _conf.get(
                    "path", ""
                )  # Websocket path, http path, quic encrypt key
            else:
                with contextlib.suppress(IndexError):
                    host = _conf.get("host", ";").split(";")[0]
                    path = _conf.get("host", ";").split(";")[1]
            tls = _conf.get("tls", "none")  # TLS
            tls_host = host
            security = _conf.get("security", "auto")
            remarks = _conf.get("ps", server)
            group = _conf.get("group", _conf.get("url_group", "N/A"))

            logger.debug(
                f"Server : {server}, Port : {port}, tls-host : {tls_host}, Path : {path}, "
                f"Type : {_type}, UUID : {uuid}, AlterId : {aid}, Network : {net}, "
                f"Host : {host}, TLS : {tls}, Remarks : {remarks}, group={group}"
            )
            return {
                "remarks": remarks,
                "server": server,
                "server_port": port,
                "id": uuid,
                "alterId": aid,
                "security": security,
                "type": _type,
                "path": path,
                "network": net,
                "tls-host": tls_host,
                "host": host,
                "tls": tls,
            }

        except Exception:
            logger.exception(f"Parse {raw_link} failed. (V2RayN Method)")
            return None

    def parse_gui_data(self, data: dict) -> list:
        sub_list = data.get("subItem", [])
        for item in data["vmess"]:
            _dict = {
                "server": item["address"],
                "server_port": item["port"],
                "id": item["id"],
                "alterId": item["alterId"],
                "security": item.get("security", "auto"),
                "type": item.get("headerType", "none"),
                "path": item.get("path", ""),
                "network": item["network"],
                "host": item.get("requestHost", ""),
                "tls": item.get("streamSecurity", ""),
                "allowInsecure": item.get("allowInsecure", ""),
                "subId": item.get("subid", ""),
                "remarks": item.get("remarks", item["address"]),
                "group": item.get("group", item.get("url_group", "N/A")),
            }
            if sub_id := _dict["subId"]:
                for sub in sub_list:
                    if sub_id == sub.get("id", ""):
                        _dict["group"] = sub.get("remarks", "N/A")
            self.__decoded_configs.append(_dict)
        return self.__decoded_configs

    def parse_gui_config(self, filename: str) -> Union[list, bool]:
        with open(filename, "r", encoding="utf-8") as f:
            try:
                config = json.load(f)
            except Exception:
                logger.exception("Not V2RayN Config.")
                return False
        return self.parse_gui_data(config)


if __name__ == "__main__":
    LINK = "vmess://eyJhZGQiOiAiczM1NC5va2dnbm9kZS50b3AiLCAiYWlkIjogIjAiLCAiaG9zdCI6ICJzMzU0Lm9rZ2dub2RlLnRvcCIsICJpZCI6ICJlNDgzMmNhMC1kNmY2LTMyYzgtODdkYS1mM2VjY2ZmZTczYzAiLCAibmV0IjogIndzIiwgInBhdGgiOiAiL3BhbmVsbXlhZG1pbiIsICJwb3J0IjogIjMzOTU0IiwgInBzIjogImdpdGh1Yi5jb20vZnJlZWZxIC0gXHU0ZTJkXHU1NmZkXHU5NjNmXHU5MWNjXHU0ZTkxIDEiLCAic2VjdXJpdHkiOiAibm9uZSIsICJ0bHMiOiAidGxzIiwgInR5cGUiOiAiIiwgInVybF9ncm91cCI6ICJ0Lm1lL3JpcGFvamllZGlhbiIsICJ2IjogIjIifQ=="
    print(ParserV2RayN().parse_subs_config(LINK))



import copy

import yaml
from loguru import logger


class ParserShadowsocksClash:
    def __init__(self, base_config: dict):
        self.__config_list: list = []
        self.__base_config = base_config

    def __get_shadowsocks_base_config(self) -> dict:
        return copy.deepcopy(self.__base_config)

    def __parse_config(self, clash_cfg: dict):
        proxies = clash_cfg.get("proxies", clash_cfg["Proxy"])
        for cfg in proxies:
            if cfg.get("type", "N/A").lower() != "ss":
                logger.info(f'Config {cfg["name"]}, type {cfg["type"]} not supported.')
                continue

            _dict = self.__get_shadowsocks_base_config()
            _dict["server"] = cfg["server"]
            _dict["server_port"] = int(cfg["port"])
            _dict["password"] = cfg["password"]
            _dict["method"] = cfg["cipher"]
            _dict["remarks"] = cfg.get("name", cfg["server"])
            _dict["group"] = cfg.get("group", "N/A")
            _dict["fast_open"] = False

            plugin = cfg.get("plugin")
            if plugin == "obfs":
                plugin = "obfs-local"
            elif plugin == "v2ray-plugin":
                logger.warning("V2Ray plugin not supported.")
                logger.info(f'Skip {_dict["group"]} - {_dict["remarks"]}')
                continue
            elif cfg.get("obfs") in ["http", "tls"]:
                plugin = "obfs-local"
            elif cfg.get("obfs"):
                logger.warning(f'Plugin {cfg.get("obfs")} not supported.')
                logger.info(f'Skip {_dict["group"]} - {_dict["remarks"]}')
                continue

            plugin_opts = ""
            p_opts = cfg.get("plugin-opts", {})
            if mode := p_opts.get("mode") or cfg.get("obfs"):
                plugin_opts += f"obfs={mode}"
            if host := p_opts.get("host") or cfg.get("obfs-host"):
                plugin_opts += f";obfs-host={host}"

            logger.debug(
                f'{_dict["group"]} - {_dict["remarks"]}\n'
                f"Plugin [{plugin}], mode [{mode}], host [{host}]"
            )

            _dict["plugin"] = plugin
            _dict["plugin_opts"] = plugin_opts
            _dict["plugin_args"] = ""
            self.__config_list.append(_dict)

        logger.debug(f"Read {len(self.__config_list)} configs.")

    def parse_subs_config(self, config) -> list:
        try:
            clash_cfg = yaml.load(config, Loader=yaml.FullLoader)
        except Exception:
            logger.exception("Not Clash Subscription.")
            return []

        self.__parse_config(clash_cfg)
        logger.debug(f"Read {len(self.__config_list)} configs.")
        return self.__config_list

    def parse_gui_config(self, filename: str) -> list:
        with open(filename, "r+", encoding="utf-8") as f:
            try:
                clash_cfg = yaml.load(f, Loader=yaml.FullLoader)
            except Exception:
                logger.exception("Not Clash config.")
                return []

        self.__parse_config(clash_cfg)
        logger.debug(f"Read {len(self.__config_list)} configs.")

        return self.__config_list



import os
import time
from typing import List, Tuple, Union

from loguru import logger
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.chrome.service import Service as ChromeService
from selenium.webdriver.common.by import By
from webdriver_manager.chrome import ChromeDriverManager


def speed_test_netflix(port: int) -> Tuple[float, float, List[float], float]:
    try:
        chrome_options = Options()
        chrome_options.add_argument("--headless")
        chrome_options.add_argument(f"--proxy-server=socks5://127.0.0.1:{port}")
        driver = webdriver.Chrome(
            service=ChromeService(ChromeDriverManager().install()),
            options=chrome_options,
        )
        driver.get("https://fast.com/")

        current_speed: Union[int, float] = 0
        max_speed: Union[int, float] = 0
        total_received: Union[int, float] = 0
        speed_list: list = []
        for i in range(60):
            time.sleep(0.5)
            current_speed = float(
                driver.find_element(By.CLASS_NAME, "speed-results-container").text
            )
            unit = driver.find_element(By.CLASS_NAME, "speed-units-container").text
            speed_list.append(current_speed * 128 * 1024)
            if unit == "Gbps":
                current_speed *= 1024
            elif unit == "Kbps":
                current_speed /= 1024
            if current_speed > max_speed:
                max_speed = current_speed
            total_received += current_speed * 128 * 1024
            print(
                "\r["
                + "=" * i
                + f"> CurrentInternetSpeed: [{current_speed / 8:.2f} MB/s]",
                end="",
            )
            done = len(driver.find_element(value="your-speed-message").text)
            if done:
                break

        logger.info(
            "\nNetflix test: "
            f"EndSpeed {current_speed / 8:.2f} MB/s, "
            f"MaxSpeed {max_speed / 8:.2f} MB/s."
        )
        driver.close()
        return (
            current_speed * 128 * 1024,
            max_speed * 128 * 1024,
            speed_list,
            total_received,
        )

    except Exception as e:
        logger.error(f"Netflix speedtest ERROR : Re-test node {str(e.args)}")
        return 0, 0, [], 0

    finally:
        os.system("taskkill /im chromedriver.exe /F")


if __name__ == "__main__":
    logger.info(speed_test_netflix(7890))



from ssrspeed.launcher.base import BaseClient


class V2Ray(BaseClient):
    def __init__(self, clients_dir: str, file: str):
        super().__init__(clients_dir, {"win": "v2ray-core", "unix": "v2ray-core"}, file)
        self._cmd: dict = {
            "win_debug": [
                f"{self._clients_dir}v2ray-core/v2ray.exe",
                "--config",
                self._config_file,
            ],
            "win": [
                f"{self._clients_dir}v2ray-core/v2ray.exe",
                "--config",
                self._config_file,
            ],
            "unix_debug": [
                f"{self._clients_dir}v2ray-core/v2ray",
                "--config",
                self._config_file,
            ],
            "unix": [
                f"{self._clients_dir}v2ray-core/v2ray",
                "--config",
                self._config_file,
            ],
        }



import socket

import socks
from loguru import logger

from ssrspeed.speedtest.method import (
    fast,
    google_ping,
    speedtestnet,
    st_asyncio,
    st_stream,
    stNF,
    stSocket,
    stYtb,
    tcp_ping,
    webpage_simulation,
)


class SpeedTestMethods:

    DEFAULT_SOCKET = socket.socket

    def __init__(self):
        self.__init_socket()

    @classmethod
    def __init_socket(cls):
        socket.socket = cls.DEFAULT_SOCKET

    async def start_test(self, **kwargs):
        address = kwargs.get("address", "127.0.0.1")
        port = kwargs.get("port", 10870)
        file_download = kwargs.get("file_download", {})
        speed_test = kwargs.get("speed_test", False)
        method = kwargs.get("method", "ST_ASYNC")
        socket_method = kwargs.get("socket_method", "SOCKET")
        st_speed_test = kwargs.get("st_speed_test", False)

        logger.info(f"Starting speed test with {method}.")
        if method == "ST_ASYNC":
            try:
                result = await st_asyncio.start(file_download, address, port)
                return result
            except Exception:
                logger.exception("")
        elif method == "SOCKET":  # Old speedtest
            try:
                if socket_method == "SOCKET":
                    result = await stSocket.speed_test_socket(
                        file_download, address, port, speed_test, st_speed_test
                    )
                    return result
                if socket_method == "YOUTUBE":
                    result = stYtb.speed_test_ytb(port)
                    return result
                if socket_method == "NETFLIX":
                    result = stNF.speed_test_netflix(port)
                    return result
            except Exception:
                logger.exception("")
        elif method == "SPEED_TEST_NET":
            try:
                socks.set_default_proxy(socks.SOCKS5, address, port)
                socket.socket = socks.socksocket
                logger.info("Initializing...")
                s = speedtestnet.Speedtest()
                logger.info("Selecting Best Server.")
                logger.info(s.get_best_server())
                logger.info("Testing Download...")
                s.download()
                result = s.results.dict()
                self.__init_socket()
                return result["download"] / 8, 0, [], 0  # bits to bytes
            except Exception:
                logger.exception("")
        elif method == "FAST":
            try:
                fast.set_proxy(address, port)
                result = fast.fast_com(verbose=True)
                self.__init_socket()
                #   print(result)
                return result, 0, [], 0
            except Exception:
                logger.exception("")
        elif method == "YOUTUBE":
            try:
                result = await stSocket.speed_test_socket(
                    file_download, address, port, speed_test, st_speed_test
                )
                return result
            except Exception:
                logger.exception("")
        else:
            raise ValueError(f"Invalid test method {method}.")
        return 0, 0, [], 0

    @staticmethod
    def start_tcp_ping(server, port):
        logger.info("Testing latency to server.")
        return tcp_ping(server, port)

    @staticmethod
    def start_google_ping(address, port):
        logger.info("Testing latency to google.")
        return google_ping(address, port)

    @staticmethod
    def start_stream_test(port, stream_cfg, outbound_ip):
        return st_stream.start_stream_test(port, stream_cfg, outbound_ip)

    @staticmethod
    def start_wps_test(wps_config, address, port):
        return webpage_simulation.start_web_page_simulation_test(
            wps_config, address, port
        )



import json
import os
import sys

_ = os.sep
CURRENT_PATH = os.path.dirname(__file__) + _
ROOT_PATH = _.join(os.path.realpath(__file__).split(_)[:-3]) + _
JSON_PATH = f"{CURRENT_PATH}paths.json"
SSR_PATH = _.join(CURRENT_PATH.split(_)[:-2]) + _
WD_PATH = os.getcwd() + _

INNER_PATH = {
    "ssrspeed": f"{SSR_PATH}",
    "static": f"{SSR_PATH}resource{_}static{_}",
    "fonts": f"{SSR_PATH}resource{_}static{_}fonts{_}",
    "logos": f"{SSR_PATH}resource{_}static{_}logos{_}",
    "templates": f"{SSR_PATH}resource{_}templates{_}",
    "ssrspeed.example.json": f"{SSR_PATH}resource{_}ssrspeed.example.json",
}


def get_path_json(work_path: str = WD_PATH) -> dict:
    work_dir = work_path if work_path.endswith(_) else work_path + _
    inner_dict = {
        "data": f"{work_dir}data{_}",
        "logs": f"{work_dir}data{_}logs{_}",
        "results": f"{work_dir}data{_}results{_}",
        "tmp": f"{work_dir}data{_}tmp{_}",
        "uploads": f"{work_dir}data{_}tmp{_}uploads{_}",
        "config.json": f"{work_dir}data{_}tmp{_}config.json",
        "ssrspeed.json": f"{work_dir}data{_}ssrspeed.json",
        "resources": f"{work_dir}resources{_}",
        "clients": f"{work_dir}resources{_}clients{_}",
        "databases": f"{work_dir}resources{_}databases{_}",
        "custom": f"{work_dir}resources{_}custom{_}",
        "resources.json": f"{work_dir}resources{_}resources.json",
    }
    inner_dict.update(INNER_PATH)
    return inner_dict


def root():
    p = ROOT_PATH
    os.chdir(p)
    sys.path.insert(0, p)


if __name__ == "__main__":
    print(CURRENT_PATH)
    print(ROOT_PATH)
    print(JSON_PATH)
    print(SSR_PATH)
    print(WD_PATH)
    print(json.dumps(get_path_json(), indent=4))



import os
import time
from typing import List, Tuple

from loguru import logger
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.chrome.service import Service as ChromeService
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.by import By
from webdriver_manager.chrome import ChromeDriverManager


def speed_test_ytb(port: int) -> Tuple[float, float, List[float], float]:
    try:
        chrome_options = Options()
        chrome_options.add_argument("--headless")
        chrome_options.add_argument(f"--proxy-server=socks5://127.0.0.1:{port}")
        chrome_options.add_argument(
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
            "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36"
        )
        driver = webdriver.Chrome(
            service=ChromeService(ChromeDriverManager().install()),
            options=chrome_options,
        )

        driver.set_window_size(3840, 2160)
        driver.get(
            "https://www.youtube.com/watch?v=mkggXE5e2yk&list=RDCMUCve7_yAZHFNipzeAGBI5t9g"
        )
        driver.find_element(By.CLASS_NAME, "ytp-large-play-button").click()
        driver.find_element(By.CLASS_NAME, "ytp-fullscreen-button").click()

        # print(driver.get_window_rect(), "\n", driver.get_window_size())
        time.sleep(0.5)
        s = driver.find_element(By.CLASS_NAME, "ytp-settings-button")
        s.click()
        logger.info(
            "Youtube view quality : "
            + driver.find_element(By.CLASS_NAME, "ytp-menu-label-secondary").text
        )
        s.click()
        t = driver.find_element(By.CLASS_NAME, "html5-main-video")
        ActionChains(driver).context_click(t).perform()
        time.sleep(0.5)
        driver.find_elements(By.CLASS_NAME, "ytp-menuitem-label")[-1].click()
        logger.info(
            "Youtube view frame : "
            + driver.find_element(By.XPATH, "//span[contains(text(),'@60')]").text
        )

        st_speed: float = 0
        max_speed: float = 0
        total_received: float = 0
        speed_list: list = []
        for i in range(20):
            time.sleep(1)
            s1 = driver.find_element(By.XPATH, "//*[contains(text(),'Kbps')]").text
            current_speed = int(s1[: s1.find(" ")])
            total_received += current_speed * 128
            speed_list.append(current_speed * 128)
            if not i:
                st_speed = current_speed
            if current_speed > max_speed:
                max_speed = current_speed
            print(
                "\r["
                + "=" * i
                + f"> [{i * 5 + 5}%/100%] [{current_speed / 8 / 1024:.2f} MB/s]",
                end="",
            )

        logger.info(
            "\nYoutube test: "
            f"StartSpeed {st_speed / 1024 / 8:.2f} MB/s, "
            f"MaxSpeed {max_speed / 8 / 1024:.2f} MB/s."
        )
        driver.close()
        return st_speed * 128, max_speed * 128, speed_list, total_received

    except Exception as e:
        logger.error(f"Youtube test ERROR : Re-testing node {str(e.args)}")
        return 0, 0, [], 0

    finally:
        os.system("taskkill /im chromedriver.exe /F")


if __name__ == "__main__":
    logger.info(speed_test_ytb(7890))



import json
import os
import shutil
import time

from loguru import logger


class ExporterWps:
    def __init__(self, result: list, results_dir: str, templates_dir: str):
        now_time = []
        for item in result:
            item["geoIP"]["inbound"]["address"] = "*.*.*.*"
            item["geoIP"]["outbound"]["address"] = "*.*.*.*"
            now_time.append(item)
        self.__results: list = now_time
        self.__results_dir: str = results_dir
        self._templates_dir: str = templates_dir

    def export(self):
        now_time = time.strftime("%Y-%m-%d-%H-%M-%S", time.localtime())
        fileloc = self.__results_dir + now_time
        res = "var res = " + json.dumps(
            self.__results, sort_keys=True, indent=4, separators=(",", ":")
        )
        if os.path.exists(fileloc):
            shutil.rmtree(fileloc)
        shutil.copytree(self._templates_dir, fileloc)
        filename = os.path.join(fileloc, "results.js")
        with open(filename, "w+", encoding="utf-8") as f:
            f.writelines(res)
        index_filename = os.path.join(fileloc, "index.html")
        with open(index_filename, "r", encoding="utf-8") as f:
            read = f.readlines()
        index = "".join(read)
        index = index.replace(r"{{ $generatedTime }}", now_time)
        with open(index_filename, "w+", encoding="utf-8") as f:
            f.writelines(index)
        logger.info(f"Web page simulation result exported as {fileloc}")


if __name__ == "__main__":
    from ssrspeed.path import ROOT_PATH, get_path_json

    key_path = get_path_json(ROOT_PATH)
    er = ExporterWps([], key_path["results"], key_path["templates"])
    er.export()



import yaml
from loguru import logger


class ParserV2RayClash:
    def __init__(self):
        self.__clash_vmess_configs: list = []
        self.__decoded_configs: list = []

    @staticmethod
    def __clash_config_convert(clash_cfg: dict) -> dict:
        server = clash_cfg["server"]
        remarks = clash_cfg.get("name", server)
        group = "N/A"
        port = int(clash_cfg["port"])
        uuid = clash_cfg["uuid"]
        aid = int(clash_cfg["alterId"])
        security = clash_cfg.get("cipher", "auto")
        tls = "tls" if "tls" in clash_cfg else ""  # TLS
        allow_insecure = bool(clash_cfg.get("skip-cert-verify", False))
        net = clash_cfg.get("network", "tcp")  # ws, tcp
        _type = clash_cfg.get("type", "none")  # Obfs type
        ws_header = clash_cfg.get("ws-headers", {})
        headers = {
            header: ws_header[header] for header in ws_header.keys() if header != "Host"
        }
        # http host, web socket host, h2 host, quic encrypt method
        host = ws_header.get("Host", "")
        tls_host = host
        # Websocket path, http path, quic encrypt key
        path = clash_cfg.get("ws-path", "")

        logger.debug(
            f"Server : {server}, Port : {port}, tls-host : {tls_host}, Path : {path}, "
            f"Type : {_type}, UUID : {uuid}, AlterId : {aid}, Network : {net}, "
            f"Host : {host}, TLS : {tls}, Remarks : {remarks}, group={group}"
        )
        return {
            "remarks": remarks,
            "group": group,
            "server": server,
            "server_port": port,
            "id": uuid,
            "alterId": aid,
            "security": security,
            "type": _type,
            "path": path,
            "allowInsecure": allow_insecure,
            "network": net,
            "headers": headers,
            "tls-host": tls_host,
            "host": host,
            "tls": tls,
        }

    def __parse_config(self, clash_cfg: dict):
        for cfg in clash_cfg["proxies"]:
            if cfg.get("type", "N/A").lower() == "vmess":
                self.__clash_vmess_configs.append(cfg)
        for cfg in self.__clash_vmess_configs:
            self.__decoded_configs.append(self.__clash_config_convert(cfg))

    def parse_subs_config(self, config) -> list:
        try:
            clash_cfg = yaml.load(config, Loader=yaml.FullLoader)
        except Exception:
            logger.exception("Not Clash config.")
            return []
        self.__parse_config(clash_cfg)
        return self.__decoded_configs

    def parse_gui_config(self, filename: str) -> list:
        with open(filename, "r+", encoding="utf-8") as f:
            try:
                clash_cfg = yaml.load(f, Loader=yaml.FullLoader)
            except Exception:
                logger.exception("Not Clash config.")
                return []
        self.__parse_config(clash_cfg)
        return self.__decoded_configs


if __name__ == "__main__":
    from ssrspeed.path import ROOT_PATH, get_path_json

    key_path = get_path_json(ROOT_PATH)
    pvc = ParserV2RayClash()
    pvc.parse_gui_config(key_path["data"] + "clash.yaml")



import time
from typing import Optional

from loguru import logger

from ssrspeed.config import ssrconfig
from ssrspeed.parser.parser import UniversalParser
from ssrspeed.result import ExportResult
from ssrspeed.result.importer import import_result
from ssrspeed.speedtest import SpeedTest


class SSRSpeedCore:
    def __init__(self):
        self.test_method: str = "ST_ASYNC"
        self.proxy_type: str = "SSR"
        self.web_mode: bool = False
        self.colors: str = "origin"
        self.sort_method: str = ""
        self.test_mode: str = "DEFAULT"

        self.__time_stamp_start: float = -1
        self.__time_stamp_stop: float = -1
        self.__parser: UniversalParser = UniversalParser()
        self.__stc: Optional[SpeedTest] = None
        self.__results: list = []
        self.__status: str = "stopped"

    def set_group(self, group: str):
        self.__parser.set_group(group)

    # Web Methods
    @staticmethod
    def web_get_colors() -> str:
        return ssrconfig["exportResult"]["colors"]

    def web_get_status(self) -> str:
        return self.__status

    @staticmethod
    def __generate_web_configs(nodes: list) -> list:
        return [{"type": node.node_type, "config": node.config} for node in nodes]

    def web_read_subscription(self, url: str) -> list:
        if parser := UniversalParser():
            urls = url.split(" ")
            parser.read_subscription(urls)
            return self.__generate_web_configs(parser.nodes)
        return []

    def web_read_config_file(self, filename: str) -> list:
        if parser := UniversalParser():
            parser.read_gui_config(filename)
            return self.__generate_web_configs(parser.nodes)
        return []

    def web_setup(self, **kwargs):
        self.test_method = kwargs.get("testMethod", "ST_ASYNC")
        self.colors = kwargs.get("colors", "origin")
        self.sort_method = kwargs.get("sortMethod", "")
        self.test_mode = kwargs.get("testMode", "TCP_PING")

    def web_set_configs(self, configs: list):
        if self.__parser:
            self.__parser.set_nodes(UniversalParser.web_config_to_node(configs))

    # Console Methods
    def console_setup(
        self,
        test_mode: str,
        test_method: str,
        color: str = "origin",
        sort_method: str = "",
        url: str = "",
        cfg_filename: str = "",
    ):
        self.test_method = test_method
        self.test_mode = test_mode
        self.sort_method = sort_method
        self.colors = color
        if self.__parser:
            if url:
                self.__parser.read_subscription(url.split(" "))
            elif cfg_filename:
                self.__parser.read_gui_config(cfg_filename)
            else:
                raise ValueError("Subscription URL or configuration file must be set!")

    def start_test(self, args):
        self.__time_stamp_start = time.time()
        self.__stc = SpeedTest(args, self.__parser, self.test_method)
        self.__status = "running"
        if self.test_mode == "DEFAULT":
            self.__stc.default_test()
        elif self.test_mode == "TCP_PING":
            self.__stc.ping_only()
        elif self.test_mode == "STREAM":
            self.__stc.stream_only()
        elif self.test_mode == "ALL":
            self.__stc.full_test()
        elif self.test_mode == "WEB_PAGE_SIMULATION":
            self.__stc.web_page_simulation()
        self.__status = "stopped"
        self.__results = self.__stc.get_result()
        self.__time_stamp_stop = time.time()
        self.__export_result()

    def clean_result(self):
        self.__results = []
        if self.__stc:
            self.__stc.reset_status()

    def get_results(self) -> list:
        return self.__results

    def web_get_results(self) -> dict:
        if self.__status == "running":
            status = "running" if self.__stc else "pending"
        else:
            status = self.__status
        return {
            "status": status,
            "current": self.__stc.get_current()
            if (self.__stc and status == "running")
            else {},
            "results": self.__stc.get_result() if self.__stc else [],
        }

    def filter_nodes(self, **kwargs: list):
        fk = kwargs.get("fk", [])
        fgk = kwargs.get("fgk", [])
        frk = kwargs.get("frk", [])
        ek = kwargs.get("ek", [])
        egk = kwargs.get("egk", [])
        erk = kwargs.get("erk", []) + ssrconfig["excludeRemarks"]
        self.__parser.filter_nodes(fk=fk, fgk=fgk, frk=frk, ek=ek, egk=egk, erk=erk)
        self.__parser.print_nodes()
        logger.info(f"{len(self.__parser.nodes)} node(s) will be tested.")

    def import_and_export(self, filename: str):
        self.__results = import_result(filename)
        self.__export_result(2)
        self.__results = []

    def __export_result(self, export_type: int = 0):
        er = ExportResult()
        er.set_time_used(self.__time_stamp_stop - self.__time_stamp_start)
        if self.test_mode == "TCP_PING":
            er.set_hide(ping=False, netflix=True, bilibili=True, StSpeed=True)
        elif self.test_mode == "STREAM":
            er.set_hide(stream=False, gping=True, StSpeed=True)
        elif self.test_mode == "ALL":
            er.set_hide(
                ntt=False,
                geoip=False,
                ping=False,
                stream=False,
                speed=False,
                port=False,
                multiplex=False,
            )
        elif self.test_mode == "WEB_PAGE_SIMULATION":
            er.export_wps_result(self.__results, export_type)
            return
        er.set_colors(self.colors)
        er.export(self.__results, export_type, self.sort_method)



from . import b64plus
from .geoip import domain2ip, ip_loc, parse_location
from .port import async_check_port, sync_check_port
from .pynat import get_ip_info
from .require import RequirementsCheck
from .system import PLATFORM

__all__ = [
    "b64plus",
    "domain2ip",
    "ip_loc",
    "parse_location",
    "get_ip_info",
    "PLATFORM",
    "async_check_port",
    "sync_check_port",
    "RequirementsCheck",
]



from ssrspeed.launcher.base import BaseClient


class Trojan(BaseClient):
    def __init__(self, clients_dir: str, file: str):
        super().__init__(clients_dir, {"win": "trojan", "unix": "trojan"}, file)
        self._cmd: dict = {
            "win_debug": [
                f"{self._clients_dir}trojan/trojan.exe",
                "--config",
                self._config_file,
            ],
            "win": [
                f"{self._clients_dir}trojan/trojan.exe",
                "--config",
                self._config_file,
            ],
            "unix_debug": [
                f"{self._clients_dir}trojan/trojan",
                "--config",
                self._config_file,
            ],
            "unix": [
                f"{self._clients_dir}trojan/trojan",
                "--config",
                self._config_file,
            ],
        }



from copy import deepcopy

import yaml
from loguru import logger


class ClashParser:
    def __init__(self, ss_base_config):
        self.__config_list: list = []
        self.__ss_base_config: dict = ss_base_config

    @property
    def config_list(self) -> list:
        return deepcopy(self.__config_list)

    def __get_shadowsocks_base_config(self) -> dict:
        return deepcopy(self.__ss_base_config)

    def __parse_shadowsocks(self, cfg: dict) -> dict:
        _config = self.__get_shadowsocks_base_config()
        _config["server"] = cfg["server"]
        _config["server_port"] = int(cfg["port"])
        _config["password"] = cfg["password"]
        _config["method"] = cfg["cipher"]
        _config["remarks"] = cfg.get("name", cfg["server"])
        _config["group"] = cfg.get("group", "N/A")
        _config["fast_open"] = False

        plugin = cfg.get("plugin")
        if plugin == "obfs":
            plugin = "obfs-local"
        elif plugin == "v2ray-plugin":
            logger.warning("V2Ray plugin not supported.")
            logger.info(f'Skip {_config["group"]} - {_config["remarks"]}')
        elif cfg.get("obfs") in ["http", "tls"]:
            plugin = "obfs-local"
        elif cfg.get("obfs"):
            logger.warning(f'Plugin {cfg.get("obfs")} not supported.')
            logger.info(f'Skip {_config["group"]} - {_config["remarks"]}')

        plugin_opts = ""
        p_opts = cfg.get("plugin-opts", {})
        if mode := p_opts.get("mode") or cfg.get("obfs"):
            plugin_opts += f"obfs={mode}"
        if host := p_opts.get("host") or cfg.get("obfs-host"):
            plugin_opts += f";obfs-host={host}"

        logger.debug(
            f'{_config["group"]} - {_config["remarks"]}\n'
            f"Plugin [{plugin}], mode [{mode}], host [{host}]"
        )

        _config["plugin"] = plugin
        _config["plugin_opts"] = plugin_opts
        _config["plugin_args"] = ""
        return _config

    @staticmethod
    def __convert_v2ray_cfg(cfg: dict) -> dict:
        server = cfg["server"]
        remarks = cfg.get("name", server)
        group = "N/A"
        port = int(cfg["port"])
        uuid = cfg["uuid"]
        aid = int(cfg["alterId"])
        security = cfg.get("cipher", "auto")
        tls = "tls" if "tls" in cfg else ""  # TLS
        allow_insecure = bool(cfg.get("skip-cert-verify", False))
        net = cfg.get("network", "tcp")  # ws, tcp
        _type = cfg.get("type", "none")  # Obfs type
        ws_header = cfg.get("ws-headers", {})
        headers = {
            header: ws_header[header] for header in ws_header.keys() if header != "Host"
        }
        # http host, web socket host, h2 host, quic encrypt method
        host = ws_header.get("Host", "")
        tls_host = host
        # Websocket path, http path, quic encrypt key
        path = cfg.get("ws-path", "")

        logger.debug(
            f"Server : {server}, Port : {port}, tls-host : {tls_host}, Path : {path}, "
            f"Type : {_type}, UUID : {uuid}, AlterId : {aid}, Network : {net}, "
            f"Host : {host}, TLS : {tls}, Remarks : {remarks}, group={group}"
        )
        return {
            "remarks": remarks,
            "group": group,
            "server": server,
            "server_port": port,
            "id": uuid,
            "alterId": aid,
            "security": security,
            "type": _type,
            "path": path,
            "allowInsecure": allow_insecure,
            "network": net,
            "headers": headers,
            "tls-host": tls_host,
            "host": host,
            "tls": tls,
        }

    @staticmethod
    def __convert_trojan_cfg(cfg: dict) -> dict:
        logger.debug(cfg)

        password = cfg["password"]
        server = cfg["server"]
        remarks = cfg.get("name", server)
        group = cfg.get("peer", "N/A")
        sni = cfg.get("sni", "")
        port = int(cfg["port"])
        allow_insecure = bool(cfg.get("skip-cert-verify", False))
        return {
            "run_type": "client",
            "local_addr": "127.0.0.1",
            "local_port": 10870,
            "remote_addr": server,
            "remote_port": port,
            "password": [password],
            "log_level": 1,
            "ssl": {
                "verify": allow_insecure,
                "verify_hostname": "true",
                "cert": "",
                "cipher": "ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:"
                "ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:"
                "ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:"
                "ECDHE-ECDSA-AES256-SHA:ECDHE-ECDSA-AES128-SHA:"
                "ECDHE-RSA-AES128-SHA:ECDHE-RSA-AES256-SHA:"
                "DHE-RSA-AES128-SHA:DHE-RSA-AES256-SHA:"
                "AES128-SHA:AES256-SHA:DES-CBC3-SHA",
                "cipher_tls13": "TLS_AES_128_GCM_SHA256:TLS_CHACHA20_POLY1305_SHA256:TLS_AES_256_GCM_SHA384",
                "sni": sni,
                "alpn": ["h2", "http/1.1"],
                "reuse_session": "true",
                "session_ticket": "false",
                "curves": "",
            },
            "tcp": {
                "no_delay": "true",
                "keep_alive": "true",
                "reuse_port": "false",
                "fast_open": "false",
                "fast_open_qlen": 20,
            },
            "group": group,
            "remarks": remarks,
            "server": server,
            "server_port": port,
        }

    def parse_config(self, clash_cfg):
        clash_cfg = yaml.load(clash_cfg, Loader=yaml.FullLoader)
        for cfg in clash_cfg["proxies"]:
            _type = cfg.get("type", "N/A").lower()
            if _type in ["ss", "ssr"]:
                ret = self.__parse_shadowsocks(cfg)
            elif _type == "vmess":
                ret = self.__convert_v2ray_cfg(cfg)
            elif _type == "trojan":
                ret = self.__convert_trojan_cfg(cfg)
            else:
                logger.error(f"Unsupported type {_type}.")
                continue
            if ret:
                self.__config_list.append({"type": _type, "config": ret})

    def parse_gui_config(self, filename):
        with open(filename, "r+", encoding="utf-8") as f:
            try:
                self.parse_config(f.read())
            except Exception:
                logger.exception("Not Clash config.")



import binascii
import copy
import json
from typing import Optional
from urllib.parse import urlparse

from loguru import logger

from ssrspeed.util import b64plus


class ParserShadowsocksBasic:
    def __init__(self, base_config: dict):
        self.__config_list: list = []
        self.__base_config: dict = base_config

    def __get_shadowsocks_base_config(self) -> dict:
        return copy.deepcopy(self.__base_config)

    def __parse_link(self, link: str) -> Optional[dict]:
        if not link.startswith("ss://"):
            logger.error(f"Not shadowsocks URL : {link}")
            return None

        try:
            _link = "ss://" + b64plus.decode(link[5:].strip("\n")).decode("utf-8")
        except binascii.Error as e:
            raise ValueError(f"Not shadowsocks basic URL : {link}") from e

        _config = self.__get_shadowsocks_base_config()
        try:
            url = urlparse(_link)
            hostname = url.hostname
            port = url.port or 443
            encryption = url.netloc.split("@")[0]
            _config["server"] = hostname
            _config["server_port"] = port
            _config["method"] = encryption.split(":")[0]
            _config["password"] = encryption.split(":")[1]
            _config["remarks"] = hostname
        except Exception:
            logger.exception(f"Invalid shadowsocks basic URL : {link}")
        return _config

    def parse_single_link(self, link: str) -> Optional[dict]:
        return self.__parse_link(link)

    def parse_subs_config(self, links: list) -> list:
        for link_ in links:
            link_ = link_.strip()
            if cfg := self.__parse_link(link_):
                self.__config_list.append(cfg)
        logger.info(f"Read {len(self.__config_list)} config(s).")
        return self.__config_list

    @staticmethod
    def __get_ssd_group(ssd_subs: list, sub_url: str) -> str:
        if not ssd_subs or not sub_url:
            return "N/A"
        return next(
            (
                item.get("airport", "N/A")
                for item in ssd_subs
                if item.get("url", "") == sub_url
            ),
            "N/A",
        )

    def parse_gui_data(self, data: dict) -> list:
        shadowsocksd_conf = False
        ssd_subs = []
        if "subscriptions" in data:
            shadowsocksd_conf = True
            ssd_subs = data["subscriptions"]

        configs = data["configs"]
        for item in configs:
            _dict = self.__get_shadowsocks_base_config()
            _dict["server"] = item["server"]
            _dict["server_port"] = int(item["server_port"])
            _dict["password"] = item["password"]
            _dict["method"] = item["method"]
            _dict["plugin"] = item.get("plugin", "")
            _dict["plugin_opts"] = item.get("plugin_opts", "")
            _dict["plugin_args"] = item.get("plugin_args", "")
            _dict["remarks"] = item.get("remarks", item["server"])
            _dict["group"] = (
                self.__get_ssd_group(ssd_subs, item.get("subscription_url", ""))
                if shadowsocksd_conf
                else item.get("group", "N/A")
            )
            _dict["fast_open"] = False
            self.__config_list.append(_dict)
        return self.__config_list

    def parse_gui_config(self, filename: str) -> list:
        with open(filename, "r", encoding="utf-8") as f:
            try:
                full_conf = json.load(f)
                self.parse_gui_data(full_conf)
            except json.decoder.JSONDecodeError:
                return []
            except Exception:
                logger.exception("Not Shadowsocks configuration file.")
                return []

        logger.info(f"Read {len(self.__config_list)} node(s).")
        return self.__config_list


if __name__ == "__main__":
    LINKS = (
        "ss://YWVzLTI1Ni1nY206aXN4Lnl0LTMxMDk1NzYxQDE5Mi4yNDEuMTkzLjI0MToxMzU0NQ==\n"
        "ss://YWVzLTEyOC1jZmI6c2hhZG93c29ja3M@156.146.38.163:443#US_13\n"
    )
    sspar = ParserShadowsocksBasic({})
    for i in LINKS.split("\n"):
        if i:
            print(sspar.parse_single_link(i))



import os.path
import re

from setuptools import find_packages, setup

this_directory = os.path.abspath(os.path.dirname(__file__))

with open(
    os.path.join(this_directory, "ssrspeed/__init__.py"), mode="r", encoding="utf-8"
) as v:
    version = re.findall(r'__version__ = "(.*?)"', v.read())[0]

with open(os.path.join(this_directory, "README.md"), mode="r", encoding="utf-8") as f:
    long_description = f.read()

with open(
    os.path.join(this_directory, "requirements.txt"), mode="r", encoding="utf-8"
) as r:
    install_requires = [i.strip() for i in r]

setup(
    name="ssrspeed",
    version=version,
    keywords=["ssr", "speed", "test"],
    url="https://github.com/OreosLab/SSRSpeedN",
    license="GPL-3.0 License",
    description="A simple tool to test nodes.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(include=["ssrspeed", "ssrspeed.*"]),
    include_package_data=True,
    python_requires=">=3.8.0",
    install_requires=install_requires,
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
    entry_points={"console_scripts": ["ssrspeed = ssrspeed.__main__:main"]},
)



import asyncio
import copy
import time

import aiohttp
from aiohttp_socks import ProxyConnector
from loguru import logger

from ssrspeed.util import parse_location

results: list = []


async def start_web_page_simulation_test(w_config, local_host, local_port):
    while results:
        results.pop()
    semaphore = asyncio.Semaphore(w_config.get("maxWorkers", 4))
    logger.info("Start web page simulation test.")
    logger.info(f"Proxy {local_host}:{local_port}")
    ip_loc = await parse_location(local_port)
    urls = copy.deepcopy(w_config.get("urls", []))
    if ip_loc[0] and ip_loc[1] == "CN":
        urls = copy.deepcopy(w_config.get("cnUrls", []))
    logger.info(f"Read {len(urls)} url(s).")
    task_list = [
        asyncio.create_task(
            execute(url=url, host=local_host, port=local_port, semaphore=semaphore)
        )
        for url in urls
    ]

    await asyncio.wait(task_list)
    return copy.deepcopy(results)


async def execute(url, host, port, semaphore):
    logger.debug(f"{asyncio.current_task().get_name()} started. Url: {url}")
    logger.info(f"Testing Url : {url}")
    res = {"url": url, "retCode": 0, "time": 0}
    try:
        start_time = time.time()
        async with aiohttp.ClientSession(
            connector=ProxyConnector(host=host, port=port),
            timeout=aiohttp.ClientTimeout(connect=10),
        ) as session, semaphore, session.get(url) as response:
            res["retCode"] = response.status
            stop_time = time.time()
            res["time"] = stop_time - start_time
            logger.info(
                f"Url: {url}, time used: {res['time']:.2f}s, code: {res['retCode']}."
            )
    except asyncio.TimeoutError:
        logger.error(f"Url: {url} timeout.")
    except ConnectionResetError:
        logger.error(f"Connection reset by peer on : {url}")
    except aiohttp.TooManyRedirects:
        logger.error(f"Too many redirects on : {url}")
    except aiohttp.ClientSSLError:
        logger.error(f"SSL Error on : {url}")
    except aiohttp.ServerDisconnectedError:
        logger.error(f"Server disconnected on : {url}")
    except Exception:
        logger.exception(f"Unknown Error on : {url}")
    finally:
        results.append(res)


if __name__ == "__main__":
    W_CONFIG = {
        "enabled": True,
        "maxWorkers": 4,
        "urls": [
            "https://www.google.com.hk",
            "https://www.youtube.com",
            "https://www.bing.com",
            "https://www.github.com",
            "https://www.microsoft.com",
        ],
        "cnUrls": [
            "https://www.baidu.com",
            "https://www.weibo.com",
            "https://www.qq.com",
        ],
    }
    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
    print(asyncio.run(start_web_page_simulation_test(W_CONFIG, "127.0.0.1", 7890)))



import requests
from loguru import logger

from ssrspeed.parser.bottom import BottomParser
from ssrspeed.parser.conf import V2RayBaseConfigs
from ssrspeed.parser.v2ray import ParserV2RayClash, ParserV2RayN, ParserV2RayQuantumult
from ssrspeed.util import b64plus


class V2RayParser(V2RayBaseConfigs, BottomParser):
    def __generate_config(self, config: dict) -> dict:
        return super().generate_config(
            config, self._get_local_config()[0], self._get_local_config()[1]
        )

    def _parse_link(self, link: str) -> dict:
        if link[:8] != "vmess://":
            logger.error(f"Unsupported link : {link}")
            return {}
        pv2rn = ParserV2RayN()
        cfg = pv2rn.parse_subs_config(link)
        if not cfg:
            pq = ParserV2RayQuantumult()
            cfg = pq.parse_subs_config(link)
        if not cfg:
            logger.error(f"Parse link {link} failed.")
            return {}
        return self.__generate_config(cfg)

    def read_subscription_config(self, url: str):
        header = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
            "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36"
        }
        rep = requests.get(url, headers=header, timeout=15)
        rep.encoding = "utf-8"
        res = rep.content.decode("utf-8")
        try:
            links_arr = (b64plus.decode(res).decode("utf-8")).split("\n")
            for link in links_arr:
                link = link.strip()
                if cfg := self._parse_link(link):
                    self._config_list.append(cfg)
        except ValueError:
            logger.info("Try V2Ray Clash Parser.")
            pv2rc = ParserV2RayClash()
            for cfg in pv2rc.parse_subs_config(res):
                self._config_list.append(self.__generate_config(cfg))
        logger.info(f"Read {len(self._config_list)} node(s).")

    def read_gui_config(self, filename):
        pv2rc = ParserV2RayClash()
        v2rnp = ParserV2RayN()
        raw_gui_configs = v2rnp.parse_gui_config(filename)
        if raw_gui_configs is False:
            logger.info("Not V2RayN Config.")
            raw_gui_configs = pv2rc.parse_gui_config(filename)
        if raw_gui_configs is False:
            logger.info("Not Clash Config.")
            logger.critical("Gui config parse failed.")
            raw_gui_configs = []
        for _dict in raw_gui_configs:
            _cfg = self.__generate_config(_dict)
            self._config_list.append(_cfg)
        logger.info(f"Read {len(self._config_list)} node(s).")



from ssrspeed.launcher.base import BaseClient


class ShadowsocksR(BaseClient):
    def __init__(self, clients_dir: str, file: str):
        super().__init__(
            clients_dir,
            {"win": "shadowsocksr-libev", "unix": "shadowsocksr-Python"},
            file,
        )
        self._cmd: dict = {
            "win_debug": [
                f"{self._clients_dir}shadowsocksr-libev/ssr-local.exe",
                "-u",
                "-v",
                "-c",
                self._config_file,
            ],
            "win": [
                f"{self._clients_dir}shadowsocksr-libev/ssr-local.exe",
                "-u",
                "-c",
                self._config_file,
            ],
            "unix_debug": [
                "python3",
                f"{self._clients_dir}shadowsocksr/shadowsocks/local.py",
                "-v",
                "-c",
                self._config_file,
            ],
            "unix": [
                "python3",
                f"{self._clients_dir}shadowsocksr/shadowsocks/local.py",
                "-c",
                self._config_file,
            ],
        }



import json
import os
import sys
import time

from loguru import logger

from ssrspeed import __version__ as version
from ssrspeed.config import generate_config_file, load_path_config, ssrconfig
from ssrspeed.download import download
from ssrspeed.path import JSON_PATH, get_path_json
from ssrspeed.shell import cli as cli_cfg
from ssrspeed.util import PLATFORM, RequirementsCheck


def init_dir(paths):
    for path in paths:
        if not os.path.exists(path):
            os.makedirs(path)


def check_dir(paths):
    flag = False  # 检测必要文件夹是否存在
    for path in paths:
        if not os.path.exists(path):
            os.makedirs(path)
            flag = True  # 发生文件夹建立行为
    if flag:
        logger.warning(
            "Please download the resource and store it in the specified location!"
        )
        sys.exit(0)


def generate_path_json(data, file):
    with open(file=file, mode="w", encoding="utf-8") as f:
        f.write(json.dumps(data, indent=4))


def get_handlers(logs_dir):
    log_file = f"{logs_dir}{time.strftime('%Y-%m-%d-%H-%M-%S', time.localtime())}.log"
    return [
        {
            "sink": sys.stdout,
            "level": "INFO",
            "format": "[<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green>][<level>{level}</level>]"
            "[<yellow>{file}</yellow>:<cyan>{line}</cyan>]: <level>{message}</level>",
            "colorize": True,  # 自定义配色
            "serialize": False,  # 以 JSON 数据格式打印
            "backtrace": True,  # 是否显示完整的异常堆栈跟踪
            "diagnose": True,  # 异常跟踪是否显示触发异常的方法或语句所使用的变量，生产环境应设为 False
            "enqueue": True,  # 默认线程安全。若想实现协程安全 或 进程安全，该参数设为 True
            "catch": True,  # 捕获异常
        },
        {
            "sink": log_file,
            "level": "INFO",
            "format": "[{time:YYYY-MM-DD HH:mm:ss.SSS}][{level}][{file}:{line}]: {message}",
            "serialize": False,
            "backtrace": True,
            "diagnose": True,
            "enqueue": True,
            "catch": True,
        },
    ]


def main():
    if PLATFORM == "Unknown":
        logger.critical("Your system is not supported. Please contact the developer.")
        sys.exit(1)

    args = cli_cfg.init(version)

    # 生成项目路径字典
    key_path = get_path_json(work_path=args.dir) if args.dir else get_path_json()

    # 下载资源文件
    if download_type := args.download:
        # 下载资源时，将自动创建目录结构
        download(
            download_type,
            PLATFORM,
            key_path["clients"],
            key_path["databases"],
            key_path["resources.json"],
        )
    else:
        # 检测外部资源目录(项目依赖项)
        check_dir([key_path["clients"], key_path["databases"]])

    # 生成项目路径 JSON 文件
    generate_path_json(key_path, JSON_PATH)
    # 导入路径 JSON 数据拷贝至缓存
    load_path_config({"VERSION": version, "path": key_path})
    # 生成配置文件并将配置文件 JSON 数据拷贝至缓存
    generate_config_file()
    # 配置日志格式及日志文件路径
    handlers = get_handlers(key_path["logs"])
    # 初始化临时文件、日志和结果集目录(非项目依赖项)
    init_dir(
        [key_path["tmp"], key_path["logs"], key_path["custom"], key_path["results"]]
    )
    if ssrconfig["fastSpeed"] is True:
        for each in handlers:
            each.update({"enqueue": False})
    # 部署日志模板
    if args.debug:
        for each in handlers:
            each.update({"level": "DEBUG"})
        logger.debug("Program running in debug mode.")
    logger.configure(handlers=handlers)
    logger.enable("__main__")

    logger.info(f"SSRSpeed {version}")

    if not args.skip_requirements_check:
        rc = RequirementsCheck(key_path["clients"], key_path["databases"])
        rc.check(PLATFORM)
    else:
        logger.warning("Requirements check skipped.")

    if args.web:

        from ssrspeed.web import start_server

        start_server(args, key_path)

    else:
        config_url = ""
        config_filename = ""

        if args.url:
            config_load_mode = 0
            config_url = args.url
        elif args.import_file:
            config_load_mode = 1
        elif args.guiConfig:
            config_load_mode = 2
            config_filename = args.guiConfig
        else:
            logger.error("No config input, exiting...")
            sys.exit(1)

        test_mode_dict = {
            "default": "DEFAULT",
            "pingonly": "TCP_PING",
            "stream": "STREAM",
            "all": "ALL",
            "wps": "WEB_PAGE_SIMULATION",
        }
        test_mode = test_mode_dict.get(args.test_mode, "DEFAULT")

        test_method_dict = {
            "stasync": "ST_ASYNC",
            "socket": "SOCKET",
            "speedtestnet": "SPEED_TEST_NET",
            "fast": "FAST",
        }
        test_method = test_method_dict.get(args.test_method, "ST_ASYNC")

        filter_keyword = args.filter
        filter_group_keyword = args.group
        filter_remark_keyword = args.remarks
        exclude_keyword = args.efilter
        exclude_group_keyword = args.egfilter
        exclude_remark_keyword = args.erfilter

        result_image_color = args.result_color

        sort_method_dict = {
            "speed": "SPEED",
            "rspeed": "REVERSE_SPEED",
            "ping": "PING",
            "rping": "REVERSE_PING",
        }
        sort_method = sort_method_dict.get(args.sort_method, "")

        logger.debug(
            f"Test mode: {test_mode}"
            f"\nTest method: {test_method}"
            f"\nFilter keyword : {filter_keyword}"
            f"\nFilter group : {filter_group_keyword}"
            f"\nFilter remark : {filter_remark_keyword}"
            f"\nExclude keyword : {exclude_keyword}"
            f"\nExclude group : {exclude_group_keyword}"
            f"\nExclude remark : {exclude_remark_keyword}"
            f"\nSort method : {sort_method}"
        )

        # SSRSpeedCore 方法需配置文件支持，故在此导入
        from ssrspeed.core import SSRSpeedCore

        sc = SSRSpeedCore()

        if args.url and config_load_mode == 0:
            sc.console_setup(
                test_mode, test_method, result_image_color, sort_method, url=config_url
            )
        elif args.import_file and config_load_mode == 1:
            import_filename = args.import_file
            sc.colors = result_image_color
            sc.sort_method = sort_method or ""
            sc.import_and_export(import_filename)
            sys.exit(0)
        else:
            sc.console_setup(
                test_mode,
                test_method,
                result_image_color,
                sort_method,
                cfg_filename=config_filename,
            )

        if args.group_override:
            sc.set_group(args.group_override)

        sc.filter_nodes(
            fk=filter_keyword,
            fgk=filter_group_keyword,
            frk=filter_remark_keyword,
            ek=exclude_keyword,
            egk=exclude_group_keyword,
            erk=exclude_remark_keyword,
        )

        sc.clean_result()

        sc.start_test(args)


if __name__ == "__main__":
    main()



import json
import os
import shutil
import sys
import zipfile
from concurrent.futures import ThreadPoolExecutor, wait

import requests
from tqdm import tqdm

from ssrspeed import __banner__ as banner


def get_terminal_size(platform):
    if platform == "Windows":
        width, height = shutil.get_terminal_size()
    else:
        try:
            width, height = os.get_terminal_size(sys.stdin.fileno())
        except (AttributeError, ValueError, OSError):
            try:
                width, height = os.get_terminal_size(sys.stdout.fileno())
            except (AttributeError, ValueError, OSError):
                width, height = (80, 25)
    return width, height


def mkdirs(paths):
    for path in paths:
        if os.path.exists(path):
            if os.path.isfile(path):
                os.remove(path)
                os.makedirs(path)
        else:
            os.makedirs(path)


def unzip(file_info):
    for file in file_info["files"]:
        zip_file = file_info["parent_path"] + file
        if os.path.exists(zip_file):
            try:
                with zipfile.ZipFile(zip_file, "r") as zips:
                    zips.extractall(file_info["parent_path"])
            except zipfile.error as e:
                print(f"{file} 解压失败, 失败原因: {e}")


def check_version(version_info, version_path):
    need_update = ["client", "database"]
    if os.path.exists(version_path):
        with open(file=version_path, mode="r+", encoding="utf-8") as f:
            old_version_info = json.load(fp=f)
            for types in version_info:
                if version_info.get(types) == old_version_info.get(types, None):
                    need_update.remove(types)
    return need_update


def update_version(version_info, version_path):
    with open(file=version_path, mode="w", encoding="utf-8") as f:
        json.dump(version_info, fp=f, indent=4)


def get_urls_info(download_type, platform, client_path, database_path):
    urls_info = []
    client_resources_url = (
        "https://api.github.com/repos/OreosLab/SSRSpeedN/releases/latest"
    )
    database_resources_url = (
        "https://api.github.com/repos/P3TERX/GeoLite.mmdb/releases/latest"
    )
    client_file = {
        "Windows": ["clients_win_64.zip"],
        "Linux": ["clients_linux_amd64.zip"],
        "MacOS": ["clients_darwin_64.zip"],
    }
    client_file_info = {
        "url": client_resources_url,
        "type": "client",
        "parent_path": client_path,
        "files": client_file[platform],
    }

    database_file_info = {
        "url": database_resources_url,
        "type": "database",
        "files": ["GeoLite2-City.mmdb", "GeoLite2-ASN.mmdb"],
        "parent_path": database_path,
    }
    if download_type == "all":
        urls_info.extend((client_file_info, database_file_info))
    elif download_type == "client":
        urls_info.append(client_file_info)
    elif download_type == "database":
        urls_info.append(database_file_info)
    return urls_info, client_file_info


def download_resource(url, headers, name, size, position, parent_path, cols):
    path = f"{parent_path}{name}"
    mode = "wb"
    current_file_size = 0
    with tqdm(
        desc=name,
        total=size,
        unit="b",
        position=position,
        colour="GREEN",
        ascii=True,
        leave=False,
        unit_scale=True,
        ncols=cols - 10,
    ) as download_bar:
        if os.path.exists(path):
            current_file_size = os.path.getsize(path)
            if current_file_size == size:
                return {"state": True, "msg": f"{name} 已存在"}
            mode = "ab"
            download_bar.update(current_file_size)
        with open(file=path, mode=mode) as f:
            for _ in range(5):
                try:
                    headers.update({"Range": f"bytes={current_file_size}-{size}"})
                    content = requests.get(
                        url=url, headers=headers, timeout=10, stream=True
                    ).iter_content(chunk_size=1024)
                    for chunk in content:
                        length = f.write(chunk)
                        download_bar.update(length)
                        current_file_size += length
                    if current_file_size == size:
                        break
                except requests.exceptions.RequestException:
                    download_bar.write(f"{name} 下载异常，尝试重新请求")
            else:
                return {"state": False, "msg": f"{name} 下载失败"}
    return {"state": True, "msg": f"已保存至 {path}"}


def executor(
    file_info,
    headers,
    need_download,
    zip_file_path,
    platform,
    new_version_info,
    version_path,
):
    task_list = []
    terminal_size = get_terminal_size(platform)
    os.system("clear" if os.name == "posix" else "cls")
    print(banner)
    with ThreadPoolExecutor() as pool:
        for kwargs in file_info:
            if kwargs["types"] in need_download:
                if not os.path.exists(kwargs["parent_path"]):
                    os.makedirs(kwargs["parent_path"])
                kwargs.pop("types")
                task_list.append(
                    pool.submit(
                        download_resource,
                        **kwargs,
                        headers=headers,
                        cols=terminal_size[0],
                    )
                )
        done, _ = wait(task_list)
        print("\n")
        for each in done:
            print(each.result().get("msg"))
        if all(each.result().get("state", False) for each in done):
            update_version(new_version_info, version_path)  # 确保文件完全下载完成，才允许写入版本控制文件
            unzip(zip_file_path)


def download(download_type, platform, client_path, database_path, version_path):
    _ = os.sep
    file_info = []
    version_info = {}
    proxy = "https://ghproxy.com/"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
        "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36"
    }
    mkdirs([client_path, database_path])
    urls_info, zip_file_path = get_urls_info(
        download_type, platform, client_path, database_path
    )
    for url_info in urls_info:
        response = requests.get(url=url_info["url"], headers=headers, timeout=10).json()
        version_info[url_info["type"]] = response["id"]
        file_info.extend(
            {
                "url": f"{proxy}{each['browser_download_url']}",
                "types": url_info["type"],
                "name": each["name"],
                "size": each["size"],
                "position": index,
                "parent_path": url_info["parent_path"],
            }
            for index, each in enumerate(response["assets"], 1)
            if each["name"] in url_info["files"]
        )
    if need_download := check_version(version_info, version_path):
        executor(
            file_info,
            headers,
            need_download,
            zip_file_path,
            platform,
            version_info,
            version_path,
        )
    else:
        print("本地资源已是最新版本")
    sys.exit(0)


if __name__ == "__main__":
    _ = os.sep
    current_path = os.getcwd() + _
    resources_path = f"{current_path}resources{_}"
    client_test_path = f"{resources_path}clients{_}"
    database_test_path = f"{resources_path}databases{_}"
    version_test_path = f"{resources_path}resources.json"
    download(
        download_type="all",
        platform="Windows",
        client_path=client_test_path,
        database_path=database_test_path,
        version_path=version_test_path,
    )



from typing import Optional

from ssrspeed.parser.bottom import BottomParser
from ssrspeed.parser.ssr import ParserShadowsocksR


class ShadowsocksRParser(BottomParser):
    def _parse_link(self, link: str) -> Optional[dict]:
        pssr = ParserShadowsocksR(self._get_shadowsocks_base_config())
        return pssr.parse_single_link(link)



def parse_qs_plus(dict_) -> dict:
    data: dict = {}
    if not isinstance(dict_, dict):
        return dict_
    for k, v in dict_.items():
        if isinstance(v, list):
            if len(v) == 0:
                data[k] = []
            elif len(v) == 1:
                data[k] = v[0]
            else:
                list_ = [parse_qs_plus(item) for item in v]
                data[k] = list_
        else:
            data[k] = v
    return data



import copy

V2RayBaseConfig: dict = {
    "remarks": "",
    "group": "N/A",
    "server": "",
    "server_port": "",
    "log": {"access": "", "error": "", "loglevel": "warning"},
    "inbounds": [
        {
            "port": 1087,
            "listen": "127.0.0.1",
            "protocol": "socks",
            "sniffing": {"enabled": True, "destOverride": ["http", "tls"]},
            "settings": {"auth": "noauth", "udp": True, "ip": None, "clients": None},
            "streamSettings": None,
        }
    ],
    "outbounds": [
        {
            "tag": "proxy",
            "protocol": "vmess",
            "settings": {
                "vnext": [
                    {
                        "address": "",
                        "port": 1,
                        "users": [
                            {"id": "", "alterId": 0, "email": "", "security": "auto"}
                        ],
                    }
                ],
                "servers": None,
                "response": None,
            },
            "streamSettings": {
                "network": "",
                "security": "",
                "tlsSettings": {},
                "tcpSettings": {},
                "kcpSettings": {},
                "wsSettings": {},
                "httpSettings": {},
                "quicSettings": {},
            },
            "mux": {"enabled": True},
        },
        {
            "tag": "direct",
            "protocol": "freedom",
            "settings": {"vnext": None, "servers": None, "response": None},
            "streamSettings": {},
            "mux": {},
        },
        {
            "tag": "block",
            "protocol": "blackhole",
            "settings": {"vnext": None, "servers": None, "response": {"type": "http"}},
            "streamSettings": {},
            "mux": {},
        },
    ],
    "dns": {},
    "routing": {"domainStrategy": "IPIfNonMatch", "rules": []},
}

tcpSettingsObject: dict = {
    "connectionReuse": True,
    "header": {
        "type": "http",
        "request": {
            "version": "1.1",
            "method": "GET",
            "path": ["/pathpath"],
            "headers": {
                "Host": ["hosthost.com", "test.com"],
                "User-Agent": [
                    "Mozilla/5.0 (Windows NT 10.0; WOW64) "
                    "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.75 Safari/537.36",
                    "Mozilla/5.0 (iPhone; CPU iPhone OS 10_0_2 like Mac OS X) "
                    "AppleWebKit/601.1 (KHTML, like Gecko) CriOS/53.0.2785.109",
                ],
                "Accept-Encoding": ["gzip, deflate"],
                "Connection": ["keep-alive"],
                "Pragma": "no-cache",
            },
        },
        "response": None,
    },
}

quicSettingsObject: dict = {
    "security": "none",
    "key": "",
    "header": {"type": "none", "request": None, "response": None},
}
httpSettingsObject: dict = {"path": "", "host": ["aes-128-gcm"]}
webSocketSettingsObject: dict = {
    "connectionReuse": True,
    "path": "",
    "headers": {"Host": ""},
}

tlsSettingsObject: dict = {"allowInsecure": True, "serverName": ""}


class V2RayBaseConfigs:
    @staticmethod
    def get_tls_object() -> dict:
        return copy.deepcopy(tlsSettingsObject)

    @staticmethod
    def get_ws_object() -> dict:
        return copy.deepcopy(webSocketSettingsObject)

    @staticmethod
    def get_http_object() -> dict:
        return copy.deepcopy(httpSettingsObject)

    @staticmethod
    def get_tcp_object() -> dict:
        return copy.deepcopy(tcpSettingsObject)

    @staticmethod
    def get_quic_object() -> dict:
        return copy.deepcopy(quicSettingsObject)

    @staticmethod
    def get_config() -> dict:
        return copy.deepcopy(V2RayBaseConfig)

    @staticmethod
    def generate_config(
        config: dict, listen: str = "127.0.0.1", port: int = 1087
    ) -> dict:
        _config = V2RayBaseConfigs.get_config()

        _config["inbounds"][0]["listen"] = listen
        _config["inbounds"][0]["port"] = port

        # Common
        _config["remarks"] = config["remarks"]
        _config["group"] = config.get("group", "N/A")
        _config["server"] = config["server"]
        _config["server_port"] = config["server_port"]

        # stream settings
        stream_settings = _config["outbounds"][0]["streamSettings"]
        stream_settings["network"] = config["network"]
        if config["network"] == "tcp":
            if config["type"] == "http":
                tcp_settings = V2RayBaseConfigs.get_tcp_object()
                tcp_settings["header"]["request"]["path"] = config["path"].split(",")
                tcp_settings["header"]["request"]["headers"]["Host"] = config[
                    "host"
                ].split(",")
                stream_settings["tcpSettings"] = tcp_settings
        elif config["network"] == "ws":
            web_socket_settings = V2RayBaseConfigs.get_ws_object()
            web_socket_settings["path"] = config["path"]
            web_socket_settings["headers"]["Host"] = config["host"]
            for k, v in config.get("headers", {}).items():
                web_socket_settings["headers"][k] = v
            stream_settings["wsSettings"] = web_socket_settings
        elif config["network"] == "h2":
            http_settings = V2RayBaseConfigs.get_http_object()
            http_settings["path"] = config["path"]
            http_settings["host"] = config["host"].split(",")
            stream_settings["httpSettings"] = http_settings
        elif config["network"] == "quic":
            quic_settings = V2RayBaseConfigs.get_quic_object()
            quic_settings["security"] = config["host"]
            quic_settings["key"] = config["path"]
            quic_settings["header"]["type"] = config["type"]
            stream_settings["quicSettings"] = quic_settings

        stream_settings["security"] = config["tls"]
        if config["tls"] == "tls":
            tls_settings = V2RayBaseConfigs.get_tls_object()
            tls_settings["allowInsecure"] = (
                config.get("allowInsecure", "false") == "true"
            )
            tls_settings["serverName"] = config.get("tls-host", "")
            stream_settings["tlsSettings"] = tls_settings

        _config["outbounds"][0]["streamSettings"] = stream_settings

        outbound = _config["outbounds"][0]["settings"]["vnext"][0]
        outbound["address"] = config["server"]
        outbound["port"] = config["server_port"]
        outbound["users"][0]["id"] = config["id"]
        outbound["users"][0]["alterId"] = config["alterId"]
        outbound["users"][0]["security"] = config["security"]
        _config["outbounds"][0]["settings"]["vnext"][0] = outbound
        return _config



import asyncio
import contextlib
import re
import socket

import aiohttp
import python_socks
from aiohttp_socks import ProxyConnector
from loguru import logger


async def parse_location(port):
    try:
        logger.info("Starting parse location.")
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
            "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36"
        }
        url = "https://api.ip.sb/geoip"
        async with aiohttp.ClientSession(
            headers=headers,
            connector=ProxyConnector(host="127.0.0.1", port=port),
            timeout=aiohttp.ClientTimeout(connect=10, sock_connect=10, sock_read=10),
        ) as session, session.get(url=url) as response:
            tmp = await response.json()
            logger.info(
                f'Server Country Code : {tmp["country_code"]}, '
                f'Continent Code : {tmp["continent_code"]}, '
                f'ISP : {tmp["organization"]}'
            )
            return (
                True,
                tmp["country_code"],
                tmp["continent_code"],
                tmp["organization"],
            )
    except asyncio.TimeoutError:
        logger.error("Parse location timeout.")
    except Exception as e:
        logger.error(f"Parse location failed: {repr(e)}")
        with contextlib.suppress(Exception):
            logger.error(response.content)
    return False, "DEFAULT", "DEFAULT", "DEFAULT"


def check_ipv4(ip: str) -> bool:
    r = re.compile(r"\b((?:25[0-5]|2[0-4]\d|[01]?\d\d?)(?:(?<!\.)\b|\.)){4}")
    rm = r.match(ip)
    return bool(rm and rm[0] == ip)


def domain2ip(domain: str) -> str:
    logger.info(f"Translating {domain} to ipv4.")
    try:
        return socket.gethostbyname(domain)
    except Exception:
        logger.exception(f"Translate {domain} to ipv4 failed.")
        return "N/A"


async def ip_loc(port):
    try:
        logger.info("Starting Geo IP.")
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
            "AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/105.0.0.0 Safari/537.36"
        }
        url = "https://api.ip.sb/geoip"
        async with aiohttp.ClientSession(
            headers=headers,
            connector=ProxyConnector(host="127.0.0.1", port=port),
            timeout=aiohttp.ClientTimeout(connect=10, sock_connect=10, sock_read=10),
        ) as session, session.get(url=url) as response:
            return await response.json()
    except asyncio.TimeoutError:
        logger.error("Geo IP Timeout.")
        return {}
    except python_socks.ProxyTimeoutError:
        logger.error("Geo IP Proxy Timeout.")
        return {}
    except ConnectionResetError:
        logger.error("Geo IP Reset.")
        return {}
    except python_socks.ProxyConnectionError:
        logger.error("Geo IP Proxy Connection Error.")
        return {}
    except aiohttp.ClientOSError:
        logger.error("Geo IP ClientOSError.")
        return {}
    except aiohttp.ServerDisconnectedError:
        logger.error("Geo IP Server disconnected.")
        return {}
    except aiohttp.ContentTypeError:
        logger.error("Geo IP Connection closed.")
        return {}
    except asyncio.IncompleteReadError:
        logger.error("Geo IP Incomplete Read.")
        return {}
    except Exception:
        logger.exception("Geo IP Failed.")
        try:
            logger.error(response.content)
        except Exception:
            return {}
    finally:
        await asyncio.sleep(0.1)


if __name__ == "__main__":
    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
    logger.info(domain2ip("www.google.com"))
    logger.info(asyncio.run(parse_location(7890)))
    logger.info(asyncio.run(ip_loc(7890)))
